@@ -50,63 +50,92 @@
 #' module <- at_least_one(module, c("p1", "p2"), name = "p_combined")
 #' print(module$node_list$p_combined$summary)
 #' @export
-at_least_one <- function(mcmodule,
-                         mc_names,
-                         name = NULL,
-                         all_suffix = NULL,
-                         prefix = NULL,
-                         summary = TRUE) {
+at_least_one <- function(
+  mcmodule,
+  mc_names,
+  name = NULL,
+  all_suffix = NULL,
+  prefix = NULL,
+  summary = TRUE
+) {
   module_name <- deparse(substitute(mcmodule))
 
   # Check if mcnodes are in mcmodule
   missing_nodes <- mc_names[!mc_names %in% names(mcmodule$node_list)]
 
   if (length(missing_nodes) > 0) {
-    stop(paste(
-      "Nodes",
+    stop(sprintf(
+      "Nodes %s not found in %s",
       paste(missing_nodes, collapse = ", "),
-      "not found in",
       module_name
     ))
   }
 
+  # Extract data_name for each node in mc_names
+  nodes_data_name <- lapply(mc_names, function(x) {
+    mcmodule$node_list[[x]][["data_name"]]
+  })
+  # Get unique, non-empty data_names
+  data_name <- unique(unlist(nodes_data_name))
+  data_name <- data_name[!is.na(data_name) & nzchar(data_name)]
 
-  nodes_data_name <- sapply(mc_names, function(x)
-    mcmodule$node_list[[x]][["data_name"]])
-  data_name <- unique(nodes_data_name)
-  nodes_dim <- sapply(mc_names, function(x)
-    dim(mcmodule$node_list[[x]][["mcnode"]])[3])
-  nodes_agg <- sapply(mc_names, function(x)
-    ! is.null(mcmodule$node_list[[x]][["agg_keys"]]))
-  nodes_keys <- lapply(mc_names, function(x)
-    mc_keys(mcmodule, x))
+  # Get the third dimension size (number of variates) for each node
+  nodes_dim <- sapply(mc_names, function(x) {
+    dim(mcmodule$node_list[[x]][["mcnode"]])[3]
+  })
+
+  # Check if each node is aggregated (has agg_keys)
+  nodes_agg <- sapply(mc_names, function(x) {
+    !is.null(mcmodule$node_list[[x]][["agg_keys"]])
+  })
+
+  # Key names that are common to all nodes
+  nodes_common_keys_names <- Reduce(
+    intersect,
+    lapply(mc_names, function(x) {
+      names(mc_keys(mcmodule, x))
+    })
+  )
+
+  # List of key values for each node, using only the common keys
+  nodes_common_keys <- lapply(mc_names, function(x) {
+    mc_keys(mcmodule, x)[nodes_common_keys_names]
+  })
+  names(nodes_common_keys) <- mc_names
+
+  # List of key values for each node
+  nodes_keys <- lapply(mc_names, function(x) {
+    mc_keys(mcmodule, x)
+  })
   names(nodes_keys) <- mc_names
 
-
+  # Initialize combined probability and keys_names vector
   p_all <- 0
   keys_names <- c()
 
   # Check that data_name, dimensions and keys are identical for all nodes
-  if (length(data_name) == 1 &&
+  if (
+    length(data_name) == 1 &&
       length(unique(nodes_dim)) == 1 &&
       all(!nodes_agg) &&
-      length(unique(nodes_keys)) == 1) {
-
-    data <- nodes_keys[[1]]
+      length(unique(nodes_common_keys)) == 1
+  ) {
+    data <- nodes_common_keys[[1]]
 
     # Loop to get the combined probability of all mcnodes
     for (i in seq_along(mc_names)) {
       mc_name <- mc_names[i]
       p_i <- mcmodule$node_list[[mc_name]][["mcnode"]]
-      keys_names <- unique(c(keys_names, names(nodes_keys[[i]])))
+      keys_names <- nodes_common_keys_names
 
       # Update combined probability
       p_all <- 1 - ((1 - p_all) * (1 - p_i))
     }
-
   } else {
     if (!length(mc_names) == 2) {
-      stop("To aggregate mc_names with different data_name or keys, provide exactly two mc_nodes")
+      stop(sprintf(
+        "To aggregate mc_names with different data_name or keys, provide exactly two mc_nodes"
+      ))
     }
 
     # Get keys for both nodes
@@ -135,9 +164,9 @@ at_least_one <- function(mcmodule,
 
   # Add prefix if provided
   if (!is.null(prefix) && prefix != "") {
-    prefix<-paste0(sub("_$","",prefix),"_")
+    prefix <- paste0(sub("_$", "", prefix), "_")
     p_all_mc_name <- paste0(prefix, sub(paste0("^", prefix), "", p_all_mc_name))
-    prefix<-sub("_$","",prefix)
+    prefix <- sub("_$", "", prefix)
   }
 
   # Add new node to module
@@ -153,9 +182,14 @@ at_least_one <- function(mcmodule,
     ),
     module = module_name,
     keys = keys_names,
-    node_expression = paste0("1-(", paste(
-      paste("(1-", mc_names, ")", sep = ""), collapse = "*"
-    ), ")"),
+    node_expression = paste0(
+      "1-(",
+      paste(
+        paste("(1-", mc_names, ")", sep = ""),
+        collapse = "*"
+      ),
+      ")"
+    ),
     scenario = data$scenario_id,
     data_name = data_name,
     prefix = prefix
@@ -163,10 +197,18 @@ at_least_one <- function(mcmodule,
 
   # Get agg keys (if nodes are aggregated)
   if (any(nodes_agg)) {
-    mcmodule$node_list[[p_all_mc_name]][["agg_keys"]] <- unique(unlist(lapply(mc_names, function(x)
-      mcmodule$node_list[[x]][["agg_keys"]])))
-    mcmodule$node_list[[p_all_mc_name]][["keep_variates"]] <- all(unlist(lapply(mc_names, function(x)
-      mcmodule$node_list[[x]][["keep_variates"]])))
+    mcmodule$node_list[[p_all_mc_name]][["agg_keys"]] <- unique(unlist(lapply(
+      mc_names,
+      function(x) {
+        mcmodule$node_list[[x]][["agg_keys"]]
+      }
+    )))
+    mcmodule$node_list[[p_all_mc_name]][["keep_variates"]] <- all(unlist(lapply(
+      mc_names,
+      function(x) {
+        mcmodule$node_list[[x]][["keep_variates"]]
+      }
+    )))
   }
 
   # Add summary if requested
@@ -185,13 +227,15 @@ at_least_one <- function(mcmodule,
 
 # Function to generate a consistent name with all_suffix, adds "all" by default
 generate_all_name <- function(mc_names, all_suffix = NULL) {
-  if (is.null(all_suffix))
+  if (is.null(all_suffix)) {
     all_suffix <- "all"
+  }
   # Check if "all" is already in any input
   if (any(grepl(paste0("_", all_suffix, "$"), mc_names))) {
-    stop("One of the mc_names already contains '",
-         paste0("_", all_suffix),
-         "' suffix")
+    stop(sprintf(
+      "One of the mc_names already contains '%s' suffix",
+      paste0("_", all_suffix)
+    ))
   }
 
   # Remove common suffixes by finding the common prefix
@@ -249,30 +293,37 @@ generate_all_name <- function(mc_names, all_suffix = NULL) {
 #' )
 #' print(imports_mcmodule$node_list$no_detect_a_agg$summary)
 #' @export
-agg_totals <- function(mcmodule,
-                       mc_name,
-                       agg_keys = NULL,
-                       agg_suffix = NULL,
-                       prefix = NULL,
-                       name = NULL,
-                       summary = TRUE,
-                       keep_variates = FALSE,
-                       agg_func = NULL) {
+agg_totals <- function(
+  mcmodule,
+  mc_name,
+  agg_keys = NULL,
+  agg_suffix = NULL,
+  prefix = NULL,
+  name = NULL,
+  summary = TRUE,
+  keep_variates = FALSE,
+  agg_func = NULL
+) {
   module_name <- deparse(substitute(mcmodule))
 
   # Check if mcnode is in mcmodule
   if (!mc_name %in% names(mcmodule$node_list)) {
-    stop(mc_name, " not found in ", module_name)
+    stop(sprintf("%s not found in %s", mc_name, module_name))
   }
 
-
-  if (!(is.null(agg_func) ||
-        agg_func %in% c("prob", "avg", "sum")))
+  if (
+    !(is.null(agg_func) ||
+      agg_func %in% c("prob", "avg", "sum"))
+  ) {
     stop("Aggregation function must be prob, avg, sum or NULL")
+  }
   if (is.null(agg_keys)) {
     agg_keys <- "scenario_id"
-    message("Keys to aggregate by not provided, using 'scenario_id' by default")
+    message(sprintf(
+      "Keys to aggregate by not provided, using 'scenario_id' by default"
+    ))
   }
+
   # Extract module name and node data
   mcnode <- mcmodule$node_list[[mc_name]][["mcnode"]]
   key_col <- mc_keys(mcmodule, mc_name, agg_keys)
@@ -280,8 +331,9 @@ agg_totals <- function(mcmodule,
 
   # Generate name for aggregated node
   agg_mc_name <- if (is.null(name)) {
-    if (is.null(agg_suffix))
+    if (is.null(agg_suffix)) {
       agg_suffix <- "agg"
+    }
     paste0(mc_name, "_", agg_suffix)
   } else if (is.null(agg_suffix)) {
     name
@@ -291,9 +343,13 @@ agg_totals <- function(mcmodule,
 
   # Add prefix if provided
   if (!is.null(prefix) && prefix != "") {
-    prefix<-paste0(sub("_$","",prefix),"_")
-    agg_mc_name <- paste0(prefix, "_" , sub(paste0("^", prefix), "", agg_mc_name))
-    prefix<-sub("_$","",prefix)
+    prefix <- paste0(sub("_$", "", prefix), "_")
+    agg_mc_name <- paste0(
+      prefix,
+      "_",
+      sub(paste0("^", prefix), "", agg_mc_name)
+    )
+    prefix <- sub("_$", "", prefix)
   }
 
   # Extract variates
@@ -315,9 +371,11 @@ agg_totals <- function(mcmodule,
     if (!is.null(agg_func) && agg_func == "avg") {
       # Calculate average value
       total_lev <- Reduce("+", variates_list[index]) / sum(index)
-    } else if ((is.null(agg_func) &&
-                grepl("_n$", mc_name)) ||
-               (!is.null(agg_func) && agg_func == "sum")) {
+    } else if (
+      (is.null(agg_func) &&
+        grepl("_n$", mc_name)) ||
+        (!is.null(agg_func) && agg_func == "sum")
+    ) {
       # Sum for counts
       total_lev <- Reduce("+", variates_list[index])
     } else {
@@ -325,7 +383,6 @@ agg_totals <- function(mcmodule,
       total_lev <- 1 - Reduce("*", inv_variates_list[index])
     }
 
-
     # Aggregate results
     if (keep_variates) {
       # One row per original variate
@@ -364,23 +421,29 @@ agg_totals <- function(mcmodule,
       paste0("Average value by: ", paste0(agg_keys, collapse = ", "))
     mcmodule$node_list[[agg_mc_name]][["node_expression"]] <-
       paste0("Average ", mc_name, " by: ", paste0(agg_keys, collapse = ", "))
-  } else if ((is.null(agg_func) &&
-              grepl("_n$", mc_name)) ||
-             (!is.null(agg_func) && agg_func == "sum")) {
+  } else if (
+    (is.null(agg_func) &&
+      grepl("_n$", mc_name)) ||
+      (!is.null(agg_func) && agg_func == "sum")
+  ) {
     # Sum for counts
     mcmodule$node_list[[agg_mc_name]][["description"]] <-
       paste0("Sum by: ", paste0(agg_keys, collapse = ", "))
     mcmodule$node_list[[agg_mc_name]][["node_expression"]] <-
-      paste0(mc_name,
-             "_1+",
-             mc_name,
-             "_2+... by: ",
-             paste0(agg_keys, collapse = ", "))
+      paste0(
+        mc_name,
+        "_1+",
+        mc_name,
+        "_2+... by: ",
+        paste0(agg_keys, collapse = ", ")
+      )
   } else {
     # Combine probabilities
     mcmodule$node_list[[agg_mc_name]][["description"]] <-
-      paste0("Combined probability assuming independence by: ",
-             paste0(agg_keys, collapse = ", "))
+      paste0(
+        "Combined probability assuming independence by: ",
+        paste0(agg_keys, collapse = ", ")
+      )
     mcmodule$node_list[[agg_mc_name]][["node_expression"]] <-
       paste0(
         "1-((1-",
@@ -404,8 +467,9 @@ agg_totals <- function(mcmodule,
   mcmodule$node_list[[agg_mc_name]][["inputs"]] <- mc_name
   mcmodule$node_list[[agg_mc_name]][["data_name"]] <- data_name
 
-  if (!is.null(prefix))
+  if (!is.null(prefix)) {
     mcmodule$node_list[[agg_mc_name]][["prefix"]] <- prefix
+  }
 
   if (summary) {
     mcmodule$node_list[[agg_mc_name]][["summary"]] <-
@@ -443,6 +507,7 @@ agg_totals <- function(mcmodule,
 #' @param agg_suffix Suffix for aggregated node names (default: "hag")
 #' @param keep_variates whether to preserve individual values (default: FALSE)
 #' @param summary Include summary statistics if TRUE (default: TRUE)
+#' @param data_name Data name used to create trials_n, subsets_n and subsets_p nodes if they don't exist in mcmodule (optional)
 #'
 #' @return
 #' Updated mcmodule object containing:
@@ -462,65 +527,260 @@ agg_totals <- function(mcmodule,
 #' )
 #' print(imports_mcmodule$node_list$no_detect_a_set$summary)
 #' @export
-trial_totals <- function(mcmodule,
-                         mc_names,
-                         trials_n,
-                         subsets_n = NULL,
-                         subsets_p = NULL,
-                         name = NULL,
-                         prefix = NULL,
-                         combine_prob = TRUE,
-                         all_suffix = NULL,
-                         level_suffix = c(trial = "trial",
-                                          subset = "subset",
-                                          set = "set"),
-                         mctable = set_mctable(),
-                         agg_keys = NULL,
-                         agg_suffix = NULL,
-                         keep_variates = FALSE,
-                         summary = TRUE) {
-
+trial_totals <- function(
+  mcmodule,
+  mc_names,
+  trials_n,
+  subsets_n = NULL,
+  subsets_p = NULL,
+  name = NULL,
+  prefix = NULL,
+  combine_prob = TRUE,
+  all_suffix = NULL,
+  level_suffix = c(trial = "trial", subset = "subset", set = "set"),
+  mctable = set_mctable(),
+  agg_keys = NULL,
+  agg_suffix = NULL,
+  keep_variates = FALSE,
+  summary = TRUE,
+  data_name = NULL
+) {
   module_name <- deparse(substitute(mcmodule))
 
   # Check if mcnodes are in mcmodule
   missing_nodes <- mc_names[!mc_names %in% names(mcmodule$node_list)]
 
   if (length(missing_nodes) > 0) {
-    stop(paste(
-      "Nodes",
+    stop(sprintf(
+      "Nodes %s not found in %s",
       paste(missing_nodes, collapse = ", "),
-      "not found in",
       module_name
     ))
   }
 
-  nodes_data_name <- lapply(mc_names, function(x) {
+  # Get data_name for all mc_nodes
+  names(mc_names) <- c(paste0("mc_name_", 1:length(mc_names)))
+  mc_trial_names <- c(
+    trials_n = trials_n,
+    subsets_n = subsets_n,
+    subsets_p = subsets_p
+  )
+  mc_inputs_names <- c(mc_names, mc_trial_names)
+
+  mc_inputs_names <- mc_inputs_names[
+    !is.null(mc_inputs_names) & mc_inputs_names != "1"
+  ]
+  nodes_data_name <- lapply(mc_inputs_names, function(x) {
     mcmodule$node_list[[x]][["data_name"]]
   })
+  names(nodes_data_name) <- mc_inputs_names
 
-  # Get the first node's data_name as reference
-  reference_data_name <- sort(nodes_data_name[[1]])
+  # For each node, get its data_names (for error/message context)
+  node_data_names <- lapply(mc_inputs_names, function(x) {
+    mcmodule$node_list[[x]][["data_name"]]
+  })
+  names(node_data_names) <- mc_inputs_names
 
-  # Check if all nodes have the same set of data_names
-  all_equal <- all(sapply(nodes_data_name, function(x) {
-    identical(sort(x), reference_data_name)
-  }))
+  # Filter out NULL, NA, and "" data_names for each node
+  filtered_node_data_names <- lapply(nodes_data_name, function(x) {
+    x[!is.null(x) & !is.na(x) & nzchar(x)]
+  })
 
-  if (!all_equal) {
-    stop("data_name is not equal for all nodes")
+  # Only keep nodes that have at least one valid data_name
+  filtered_node_data_names <- Filter(
+    function(x) length(x) > 0,
+    filtered_node_data_names
+  )
+
+  # Check if all remaining nodes have the same set of data_names
+  all_equal <- length(unique(lapply(filtered_node_data_names, function(x) {
+    paste(sort(x), collapse = ",")
+  }))) ==
+    1
+
+  # All unique data_names across all nodes
+  all_data_names <- unique(unlist(filtered_node_data_names))
+
+  # Calculate combined probability for all nodes if requested
+  # (if more than one node is provided)
+  if (combine_prob && length(mc_names) > 1) {
+    mcmodule <- at_least_one(
+      mcmodule = mcmodule,
+      mc_names = mc_names,
+      name = name,
+      all_suffix = all_suffix,
+      prefix = prefix,
+      summary
+    )
+
+    # Generate name for combined node
+    p_all_mc_name <- if (is.null(name)) {
+      generate_all_name(mc_names, all_suffix = all_suffix)
+    } else if (is.null(all_suffix)) {
+      name
+    } else {
+      paste0(name, "_", all_suffix)
+    }
+
+    # Update module name metadata (defaults to mcmodule)
+    mcmodule$node_list[[p_all_mc_name]][["module"]] <- module_name
+    # mc_match if several data names are provided
+    if (!all_equal) {
+      for (i in seq_along(mc_names)) {
+        mc_match_i <- mc_match(mcmodule, p_all_mc_name, mc_names[i])[[2]]
+        mc_name_i <- paste0(names(mc_names)[i], "_mc")
+        assign(mc_name_i, mc_match_i)
+      }
+    }
+    ref_mc_name <- p_all_mc_name
+    mc_names <- c(mc_names, mc_name_all = p_all_mc_name)
+  } else {
+    ref_mc_name <- mc_names[1]
+    if (!combine_prob && length(mc_names) > 1) {
+      message(sprintf(
+        "Using '%s' as reference node for mc_match",
+        ref_mc_name
+      ))
+    }
   }
 
-  data_name <- reference_data_name
+  # Determine which data_name to use
+  if (!all_equal) {
+    #If a combined probability node was created, use its data_name
+    if (!is.null(mcmodule$node_list[[ref_mc_name]])) {
+      ref_data_names <- mcmodule$node_list[[ref_mc_name]][["data_name"]]
+      if (length(ref_data_names) > 1) {
+        if (!is.null(data_name)) {
+          # User provided data_name: check it exists in any node
+          if (!data_name %in% all_data_names) {
+            stop(sprintf(
+              "Provided data_name '%s' not found in available data_names for nodes: %s",
+              data_name,
+              paste(all_data_names, collapse = ", ")
+            ))
+          }
+          ref_data_name <- data_name
+          message(sprintf(
+            "Using data_name '%s' for node creation.",
+            ref_data_name
+          ))
+        } else {
+          # Default to the last available data_name
+          ref_data_name <- ref_data_names[length(ref_data_names)]
+          # Indicate which nodes have which data_names
+          node_names_with_multiple <- names(node_data_names)[sapply(
+            node_data_names,
+            function(x) length(x) > 1
+          )]
+          msg <- sprintf(
+            "data_name is not equal for all nodes, using data_name '%s' for node creation (can be manually set with data_name argument).",
+            ref_data_name
+          )
+          if (length(node_names_with_multiple) > 0) {
+            msg <- paste0(
+              msg,
+              " Nodes with multiple data_names: ",
+              paste(node_names_with_multiple, collapse = ", "),
+              "."
+            )
+          }
+          message(sprintf("%s", msg))
+        }
+      } else {
+        ref_data_name <- ref_data_names
+      }
+      # If no combined probability node, handle data_name selection
+    } else {
+      if (!is.null(data_name)) {
+        # User provided data_name: check it exists in any node
+        if (!data_name %in% all_data_names) {
+          stop(sprintf(
+            "Provided data_name '%s' not found in available data_names for nodes: %s",
+            data_name,
+            paste(all_data_names, collapse = ", ")
+          ))
+        }
+        ref_data_name <- data_name
+        message(sprintf(
+          "Using data_name '%s' for node creation.",
+          ref_data_name
+        ))
+      } else {
+        # Default to the last available data_name
+        ref_data_name <- all_data_names[length(all_data_names)]
+        # Indicate which nodes have which data_names
+        node_names_with_multiple <- names(node_data_names)[sapply(
+          node_data_names,
+          function(x) length(x) > 1
+        )]
+        msg <- sprintf(
+          "data_name is not equal for all nodes, using data_name '%s' for node creation (can be manually set with data_name argument).",
+          ref_data_name
+        )
+        if (length(node_names_with_multiple) > 0) {
+          msg <- paste0(
+            msg,
+            " Nodes with multiple data_names: ",
+            paste(node_names_with_multiple, collapse = ", "),
+            "."
+          )
+        }
+        message(sprintf("%s", msg))
+      }
+    }
+  } else {
+    # All nodes have the same data_name(s)
+    # If user provided a valid data_name, use it; otherwise fall back to current logic
+    if (!is.null(data_name)) {
+      if (data_name %in% all_data_names) {
+        ref_data_name <- data_name
+        message(sprintf(
+          "Using provided data_name '%s' for node creation.",
+          ref_data_name
+        ))
+      } else {
+        stop(sprintf(
+          "Provided data_name '%s' not found in available data_names: '%s'.",
+          data_name,
+          paste(all_data_names, collapse = "', '")
+        ))
+      }
+    }
 
-  if (!is.null(name) &&
+    if (!exists("ref_data_name")) {
+      if (length(all_data_names) > 1) {
+        node_names_with_multiple <- names(node_data_names)[sapply(
+          node_data_names,
+          function(x) length(x) > 1
+        )]
+        ref_data_name <- all_data_names[length(all_data_names)]
+        message(sprintf(
+          "mcnodes have multiple data_name ('%s') for node(s) '%s', using '%s' for node creation (can be manually set with data_name argument)",
+          paste(all_data_names, collapse = "', '"),
+          paste(node_names_with_multiple, collapse = ", "),
+          ref_data_name
+        ))
+      } else {
+        ref_data_name <- all_data_names
+      }
+    }
+  }
+
+  if (
+    !is.null(name) &&
       length(mc_names) > 1 &&
-      !combine_prob)
-    stop("name argument can only be used when mc_names length is 1 or when combine_prob is TRUE")
+      !combine_prob
+  ) {
+    stop(sprintf(
+      "name argument can only be used when mc_names length is 1 or when combine_prob is TRUE"
+    ))
+  }
 
-  if (!all(names(level_suffix) %in% c("trial", "subset", "set")))
-    stop(
+  if (!all(names(level_suffix) %in% c("trial", "subset", "set"))) {
+    stop(sprintf(
       "Suffixes for each hierarchical level must be defined as a named vector with the following structure: c(trial = '...', subset = '...', set = '...')"
-    )
+    ))
+  }
 
   # Fix missing level suffixes
   missing_suffixes <- setdiff(c("trial", "subset", "set"), names(level_suffix))
@@ -528,26 +788,34 @@ trial_totals <- function(mcmodule,
     level_suffix[suffix] <- suffix
   }
 
-  hag_suffix <- if(is.null(agg_suffix)||agg_suffix=="") "hag" else agg_suffix
+  hag_suffix <- if (is.null(agg_suffix) || agg_suffix == "") {
+    "hag"
+  } else {
+    agg_suffix
+  }
 
-  data <- mcmodule$data[[data_name]]
+  data <- mcmodule$data[[ref_data_name]]
 
   # Function for individual mcnode creation and processing
-  process_trial_mcnode <- function(mc_name,
-                             node_type,
-                             mcmodule,
-                             data,
-                             module_name,
-                             agg_keys,
-                             hag_suffix,
-                             mctable,
-                             keep_variates,
-                             agg_func = NULL) {
+  process_trial_mcnode <- function(
+    mc_name,
+    node_type,
+    mcmodule,
+    data,
+    module_name,
+    agg_keys,
+    hag_suffix,
+    mctable,
+    keep_variates,
+    ref_data_name,
+    agg_func = NULL
+  ) {
     if (mc_name %in% names(mcmodule$node_list)) {
       mc_node <- mcmodule$node_list[[mc_name]][["mcnode"]]
     } else {
-      if (!mc_name %in% mctable$mcnode)
-        stop(mc_name, " not found in mctable")
+      if (!mc_name %in% mctable$mcnode) {
+        stop(sprintf("%s not found in mctable", mc_name))
+      }
 
       mc_row <- mctable[mctable$mcnode %in% mc_name, ]
 
@@ -561,13 +829,17 @@ trial_totals <- function(mcmodule,
       mcmodule$node_list[[mc_name]][["inputs_col"]] <- inputs_col
 
       if (!is.na(mc_row$mc_func)) {
-        mcmodule$node_list[[mc_name]][["mc_func"]] <- as.character(mc_row$mc_func)
+        mcmodule$node_list[[mc_name]][["mc_func"]] <- as.character(
+          mc_row$mc_func
+        )
       }
 
-      mcmodule$node_list[[mc_name]][["description"]] <- as.character(mc_row$description)
+      mcmodule$node_list[[mc_name]][["description"]] <- as.character(
+        mc_row$description
+      )
       mcmodule$node_list[[mc_name]][["type"]] <- node_type
       mcmodule$node_list[[mc_name]][["module"]] <- module_name
-      mcmodule$node_list[[mc_name]][["data_name"]] <- data_name
+      mcmodule$node_list[[mc_name]][["data_name"]] <- ref_data_name
       mcmodule$node_list[[mc_name]][["mcnode"]] <- mc_node
       mcmodule$node_list[[mc_name]][["mc_func"]] <- mc_row$mc_func
 
@@ -595,11 +867,12 @@ trial_totals <- function(mcmodule,
           invokeRestart("muffleMessage")
         }
       )
-      if (!all(grepl("variates per group for", messages)))
-        message(messages)
+      if (!all(grepl("variates per group for", messages))) {
+        message(sprintf("%s", paste(messages, collapse = "; ")))
+      }
       # Change mcnode name to agg version name
       mc_name_name <- deparse(substitute(mc_name))
-      agg_mc_name <-  paste0(mc_name, "_", hag_suffix)
+      agg_mc_name <- paste0(mc_name, "_", hag_suffix)
       assign(mc_name_name, agg_mc_name, envir = parent.frame())
       # Add agg_keys to metadata
       mcmodule$node_list[[agg_mc_name]][["agg_keys"]] <- agg_keys
@@ -607,11 +880,11 @@ trial_totals <- function(mcmodule,
       mcmodule$node_list[[agg_mc_name]][["module"]] <- module_name
       mcmodule$node_list[[agg_mc_name]][["keep_variates"]] <- keep_variates
     }
+
     return(mcmodule)
   }
 
   # Process all nodes
-
   mcmodule <- process_trial_mcnode(
     trials_n,
     "trials_n",
@@ -621,10 +894,16 @@ trial_totals <- function(mcmodule,
     agg_keys,
     hag_suffix,
     mctable,
-    keep_variates
+    keep_variates,
+    ref_data_name
   )
-  trials_n_mc <- mcmodule$node_list[[trials_n]][["mcnode"]]
 
+  # mc_match if several data names are provided
+  trials_n_mc <- if (!all_equal) {
+    mc_match(mcmodule, ref_mc_name, trials_n)[[2]]
+  } else {
+    mcmodule$node_list[[trials_n]][["mcnode"]]
+  }
 
   # If subsets_n is NULL, defaults to 1
   if (is.null(subsets_n)) {
@@ -642,9 +921,17 @@ trial_totals <- function(mcmodule,
       hag_suffix,
       mctable,
       keep_variates,
+      ref_data_name,
       agg_func = "avg"
     )
-    subsets_n_mc <- mcmodule$node_list[[subsets_n]][["mcnode"]]
+
+    # mc_match if several data names are provided
+    subsets_n_mc <- if (!all_equal) {
+      mc_match(mcmodule, ref_mc_name, subsets_n)[[2]]
+    } else {
+      mcmodule$node_list[[subsets_n]][["mcnode"]]
+    }
+
     hierarchical_n <- TRUE
   }
 
@@ -666,51 +953,34 @@ trial_totals <- function(mcmodule,
       hag_suffix,
       mctable,
       keep_variates,
+      ref_data_name,
       agg_func = "avg"
     )
-    subsets_p_mc <- mcmodule$node_list[[subsets_p]][["mcnode"]]
+
+    # mc_match if several data names are provided
+    subsets_p_mc <- if (!all_equal) {
+      mc_match(mcmodule, ref_mc_name, subsets_p)[[2]]
+    } else {
+      mcmodule$node_list[[subsets_p]][["mcnode"]]
+    }
+
     hierarchical_p <- TRUE
   }
 
-  # Calculate combined probability for all nodes if requested
-  # (if more than one node is provided)
-  if (combine_prob && length(mc_names) > 1) {
-    mcmodule <- at_least_one(
-      mcmodule = mcmodule,
-      mc_names = mc_names,
-      name = name,
-      all_suffix = all_suffix,
-      prefix = prefix,
-      summary
-    )
-
-    # Generate name for combined node
-    p_all_mc_name <- if (is.null(name)) {
-      generate_all_name(mc_names, all_suffix = all_suffix)
-    } else if (is.null(all_suffix)) {
-      name
-    } else {
-      paste0(name, "_", all_suffix)
-    }
-
-    # Update module name metadata (defaults to mcmodule)
-    mcmodule$node_list[[p_all_mc_name]][["module"]] <- module_name
-
-    mc_names <- c(mc_names, p_all_mc_name)
-  }
-
   # Helper function to add metadata to nodes
-  add_mc_metadata <- function(node_list,
-                              name,
-                              value,
-                              params,
-                              description,
-                              expression,
-                              type = "total",
-                              keys_names,
-                              agg_keys,
-                              total_type,
-                              keep_variates) {
+  add_mc_metadata <- function(
+    node_list,
+    name,
+    value,
+    params,
+    description,
+    expression,
+    type = "total",
+    keys_names,
+    agg_keys,
+    total_type,
+    keep_variates
+  ) {
     node_list[[name]] <- list(
       mcnode = value,
       param = params,
@@ -721,7 +991,7 @@ trial_totals <- function(mcmodule,
       module = module_name,
       keys = keys_names,
       scenario = data$scenario_id,
-      data_name = data_name,
+      data_name = all_data_names,
       prefix = prefix,
       total_type = total_type
     )
@@ -738,48 +1008,54 @@ trial_totals <- function(mcmodule,
   calculations <- list(
     trial = list(
       prob = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc)
-          p_a * subsets_p_mc,
-        description = paste0("Probability of one %s trial (", level_suffix[["trial"]], ")"),
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
+          p_a * subsets_p_mc
+        },
+        description = paste0(
+          "Probability of one %s trial (",
+          level_suffix[["trial"]],
+          ")"
+        ),
         suffix = paste0("_", level_suffix[["trial"]]),
-        expression = function(mc_name, subsets_p)
+        expression = function(mc_name, subsets_p) {
           paste0(subsets_p, "*", mc_name)
+        }
       ),
       num = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc)
-          mcnode_na_rm(p_a / p_a, 1),
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
+          mcnode_na_rm(p_a / p_a, 1)
+        },
         description = paste0("One %s trials (", level_suffix[["trial"]], ")"),
         suffix = paste0("_", level_suffix[["trial"]], "_n"),
-        expression = function(mc_name)
+        expression = function(mc_name) {
           paste0("mcnode_na_rm(", mc_name, "/", mc_name, ", 1)")
+        }
       )
     ),
     subset = list(
       prob = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc)
-          1 - (1 - subsets_p_mc * (1 - (1 - p_a) ^ trials_n_mc)),
-        description = paste0("Probability of at least one %s in a subset (", level_suffix[["subset"]], ")"),
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
+          1 - (1 - subsets_p_mc * (1 - (1 - p_a)^trials_n_mc))
+        },
+        description = paste0(
+          "Probability of at least one %s in a subset (",
+          level_suffix[["subset"]],
+          ")"
+        ),
         suffix = paste0("_", level_suffix[["subset"]]),
-        expression = function(mc_name, trials_n, subsets_p)
+        expression = function(mc_name, trials_n, subsets_p) {
           paste0("1-(1-", subsets_p, "*(1-(1-", mc_name, ")^", trials_n, "))")
+        }
       ),
       num = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc) {
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
           p_a * trials_n_mc * subsets_p_mc
         },
-        description = paste0("Expected number of %s in a subset (", level_suffix[["subset"]], ")"),
+        description = paste0(
+          "Expected number of %s in a subset (",
+          level_suffix[["subset"]],
+          ")"
+        ),
         suffix = paste0("_", level_suffix[["subset"]], "_n"),
         expression = function(mc_name, trials_n, subsets_p) {
           paste0(mc_name, "*", trials_n, "*", subsets_p)
@@ -788,33 +1064,37 @@ trial_totals <- function(mcmodule,
     ),
     set = list(
       prob = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc) {
-          1 - (1 - subsets_p_mc * (1 - (1 - p_a) ^ trials_n_mc)) ^ subsets_n_mc
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
+          1 - (1 - subsets_p_mc * (1 - (1 - p_a)^trials_n_mc))^subsets_n_mc
         },
-        description = paste0("Probability of at least one %s in a set (", level_suffix[["set"]], ")"),
+        description = paste0(
+          "Probability of at least one %s in a set (",
+          level_suffix[["set"]],
+          ")"
+        ),
         suffix = paste0("_", level_suffix[["set"]]),
         expression = function(mc_name, trials_n, subsets_n, subsets_p) {
-          paste0("1-(1-",
-                 subsets_p,
-                 "*(1-(1-",
-                 mc_name,
-                 ")^",
-                 trials_n,
-                 "))^",
-                 subsets_n)
+          paste0(
+            "1-(1-",
+            subsets_p,
+            "*(1-(1-",
+            mc_name,
+            ")^",
+            trials_n,
+            "))^",
+            subsets_n
+          )
         }
       ),
       num = list(
-        formula = function(p_a,
-                           trials_n_mc,
-                           subsets_n_mc,
-                           subsets_p_mc) {
+        formula = function(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc) {
           p_a * trials_n_mc * subsets_p_mc * subsets_n_mc
         },
-        description = paste0("Expected number of %s in a set (", level_suffix[["set"]], ")"),
+        description = paste0(
+          "Expected number of %s in a set (",
+          level_suffix[["set"]],
+          ")"
+        ),
         suffix = paste0("_", level_suffix[["set"]], "_n"),
         expression = function(mc_name, trials_n, subsets_n, subsets_p) {
           paste0(mc_name, "*", trials_n, "*", subsets_p, "*", subsets_n)
@@ -844,13 +1124,16 @@ trial_totals <- function(mcmodule,
           invokeRestart("muffleMessage")
         }
       )
-      if (!all(grepl("variates per group for", messages)))
-        message(messages)
+      if (!all(grepl("variates per group for", messages))) {
+        message(sprintf("%s", paste(messages, collapse = "; ")))
+      }
 
       # Generate name for aggregated node
       if (!is.null(name) && length(mc_names) == 1) {
         agg_mc_name <- paste0(name, "_", hag_suffix)
-        names(mcmodule$node_list)[names(mcmodule$node_list) %in% paste0(mc_name, "_", hag_suffix)] <-
+        names(mcmodule$node_list)[
+          names(mcmodule$node_list) %in% paste0(mc_name, "_", hag_suffix)
+        ] <-
           agg_mc_name
       } else {
         agg_mc_name <- paste0(mc_name, "_", hag_suffix)
@@ -866,33 +1149,48 @@ trial_totals <- function(mcmodule,
       mcmodule$node_list[[mc_name]][["keep_variates"]] <- keep_variates
 
       # Update keys_names if it does not keep all variates
-      if (!keep_variates)
+      if (!keep_variates) {
         keys_names <- agg_keys
+      }
     } else {
       keys_names <- mcmodule$node_list[[mc_name]][["keys"]]
     }
 
-    p_a <- mcmodule$node_list[[mc_name]][["mcnode"]]
+    if (!all_equal && mc_name %in% mc_inputs_names) {
+      mc_name_matched <- paste0(
+        names(mc_inputs_names)[mc_inputs_names %in% mc_name],
+        "_mc"
+      )
+      p_a <- get(mc_name_matched)
+    } else {
+      p_a <- mcmodule$node_list[[mc_name]][["mcnode"]]
+    }
 
     # If no combined (all) probabilities use new name,
     # else, it was already generated in at_least_one
     # Remove prefix (to avoid prefix duplication)
-    prefix<-paste0(sub("_$","",prefix),"_")
-    mc_name_no_prefix  <- if (length(mc_names) == 1 &&
-                              !is.null(name)) {
-      if(!is.null(agg_keys)){
+    prefix <- paste0(sub("_$", "", prefix), "_")
+    mc_name_no_prefix <- if (
+      length(mc_names) == 1 &&
+        !is.null(name)
+    ) {
+      if (!is.null(agg_keys)) {
         sub(paste0("^", prefix), "", agg_mc_name)
-      }else{
+      } else {
         sub(paste0("^", prefix), "", name)
       }
-    } else{
-       sub(paste0("^", prefix), "", mc_name)
+    } else {
+      sub(paste0("^", prefix), "", mc_name)
     }
-    prefix<-sub("_$","",prefix)
+    prefix <- sub("_$", "", prefix)
 
     # Remove hag_suffix if agg_suffix==""
-    if(!is.null(agg_suffix)&&agg_suffix==""){
-      mc_name_no_prefix<- sub(paste0("_",hag_suffix,"$"), "", mc_name_no_prefix)
+    if (!is.null(agg_suffix) && agg_suffix == "") {
+      mc_name_no_prefix <- sub(
+        paste0("_", hag_suffix, "$"),
+        "",
+        mc_name_no_prefix
+      )
     }
 
     # Process levels
@@ -913,22 +1211,24 @@ trial_totals <- function(mcmodule,
     for (level in all_levels) {
       # Process probability and number calculations
       for (calc_type in c("prob", "num")) {
-        if (level == "trial" && calc_type == "num")
+        if (level == "trial" && calc_type == "num") {
           next
+        }
         calc <- calculations[[level]][[calc_type]]
 
         new_mc_name <- if (!is.null(prefix) && prefix != "") {
           paste0(prefix, "_", mc_name_no_prefix, calc$suffix)
         } else {
           paste0(mc_name_no_prefix, calc$suffix)
-
         }
 
         # Calculate value based on level
         value <- calc$formula(p_a, trials_n_mc, subsets_n_mc, subsets_p_mc)
-        total_type <- paste(ifelse(multilevel, "multilevel", "single level"),
-                            level,
-                            calc_type)
+        total_type <- paste(
+          ifelse(multilevel, "multilevel", "single level"),
+          level,
+          calc_type
+        )
 
         # Create node and add metadata
         mcmodule$node_list <- add_mc_metadata(
@@ -936,10 +1236,11 @@ trial_totals <- function(mcmodule,
           name = new_mc_name,
           value = value,
           params = if (level == "trial") {
-            if (calc_type == "prob")
+            if (calc_type == "prob") {
               c(mc_name, subsets_p)
-            else
+            } else {
               c()
+            }
           } else if (level == "subset") {
             c(mc_name, trials_n, subsets_p)
           } else {
