@@ -3,8 +3,9 @@ suppressMessages({
     # Create test module
     test_module <- list(
       node_list = list(
-        p1 = list(
-          mcnode = mcstoc(runif,
+        p_1 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.1, 0.2, 0.3), type = "0", nvariates = 3),
             max = mcdata(c(0.2, 0.3, 0.4), type = "0", nvariates = 3),
             nvariates = 3
@@ -12,8 +13,9 @@ suppressMessages({
           data_name = "test_data",
           keys = c("category")
         ),
-        p2 = list(
-          mcnode = mcstoc(runif,
+        p_2 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.5, 0.6, 0.7), type = "0", nvariates = 3),
             max = mcdata(c(0.6, 0.7, 0.8), type = "0", nvariates = 3),
             nvariates = 3
@@ -31,22 +33,23 @@ suppressMessages({
     )
 
     # Test basic functionality
-    result <- at_least_one(test_module, c("p1", "p2"), name = "p_combined")
+    result <- at_least_one(test_module, c("p_1", "p_2"), name = "p_combined")
 
     # Check node attributes
     expect_equal(result$node_list$p_combined$type, "total")
-    expect_equal(result$node_list$p_combined$param, c("p1", "p2"))
+    expect_equal(result$node_list$p_combined$param, c("p_1", "p_2"))
 
     # Test error on missing nodes
-    expect_error(at_least_one(test_module, c("p1", "missing")),"not found")
+    expect_error(at_least_one(test_module, c("p_1", "missing")), "not found")
   })
 
   test_that("at_least_one match works", {
     # Create test module
     test_module <- list(
       node_list = list(
-        p1 = list(
-          mcnode = mcstoc(runif,
+        p_1 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.1, 0.2, 0.3), type = "0", nvariates = 3),
             max = mcdata(c(0.2, 0.3, 0.4), type = "0", nvariates = 3),
             nvariates = 3
@@ -54,8 +57,9 @@ suppressMessages({
           data_name = "data_x",
           keys = c("category")
         ),
-        p2 = list(
-          mcnode = mcstoc(runif,
+        p_2 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.5, 0.6, 0.7), type = "0", nvariates = 3),
             max = mcdata(c(0.6, 0.7, 0.8), type = "0", nvariates = 3),
             nvariates = 3
@@ -77,24 +81,31 @@ suppressMessages({
     )
 
     # Test basic functionality
-    result <- at_least_one(test_module, c("p1", "p2"), name = "p_combined")
+    result <- at_least_one(test_module, c("p_1", "p_2"), name = "p_combined")
 
     # Check node attributes
     expect_equal(result$node_list$p_combined$type, "total")
-    expect_equal(result$node_list$p_combined$param, c("p1", "p2"))
+    expect_equal(result$node_list$p_combined$param, c("p_1", "p_2"))
 
     # Verify expected keys_xy
-    expect_equal(result$node_list$p_combined$summary$category, c("A", "B", "C", "B", "B"))
+    expect_equal(
+      result$node_list$p_combined$summary$category,
+      c("A", "B", "C", "B", "B")
+    )
 
     # Test error on missing nodes
-    expect_error(at_least_one(test_module, c("p1", "missing")),"not found")
+    expect_error(at_least_one(test_module, c("p_1", "missing")), "not found")
   })
 
   test_that("generate_all_name works", {
     # Basic functionality
     expect_equal(generate_all_name(c("test_a", "test_b")), "test_all")
     expect_equal(
-      generate_all_name(c("good_special_a", "good_special_b", "good_special_top")),
+      generate_all_name(c(
+        "good_special_a",
+        "good_special_b",
+        "good_special_top"
+      )),
       "good_special_all"
     )
 
@@ -113,8 +124,9 @@ suppressMessages({
     # Create test data
     test_module <- list(
       node_list = list(
-        p1 = list(
-          mcnode = mcstoc(runif,
+        p_1 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.1, 0.2, 0.3), type = "0", nvariates = 3),
             max = mcdata(c(0.2, 0.3, 0.4), type = "0", nvariates = 3),
             nvariates = 3
@@ -122,8 +134,9 @@ suppressMessages({
           data_name = "test_data",
           keys = c("category")
         ),
-        p2 = list(
-          mcnode = mcstoc(runif,
+        p_2 = list(
+          mcnode = mcstoc(
+            runif,
             min = mcdata(c(0.5, 0.6, 0.7), type = "0", nvariates = 3),
             max = mcdata(c(0.6, 0.7, 0.8), type = "0", nvariates = 3),
             nvariates = 3
@@ -141,24 +154,23 @@ suppressMessages({
     )
 
     # Test default agg
-    result <- agg_totals(test_module, "p1")
+    result <- agg_totals(test_module, "p_1")
 
     expect_equal(
-      result$node_list[["p1_agg"]]$description,
+      result$node_list[["p_1_agg"]]$description,
       "Combined probability assuming independence by: scenario_id"
     )
 
     # Test aggregation methods
-    result_sum <- agg_totals(test_module, "p1", agg_func = "sum")
+    result_sum <- agg_totals(test_module, "p_1", agg_func = "sum")
     expect_equal(
-      result_sum$node_list[["p1_agg"]]$description,
+      result_sum$node_list[["p_1_agg"]]$description,
       "Sum by: scenario_id"
     )
 
-
-    result_avg <- agg_totals(test_module, "p1", agg_func = "avg")
+    result_avg <- agg_totals(test_module, "p_1", agg_func = "avg")
     expect_equal(
-      result_avg$node_list[["p1_agg"]]$description,
+      result_avg$node_list[["p_1_agg"]]$description,
       "Average value by: scenario_id"
     )
 
@@ -173,31 +185,34 @@ suppressMessages({
     # - p_1_x and p_1_y: Two probability nodes with uniform distribution
     # - p_2: Another probability node for subset calculations
     # - times_n: Number of trials for each category/scenario combination
-    mcmodule<-list(
+    mcmodule <- list(
       node_list = list(
         p_1_x = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.1, 0.2, 0.3, 0.5), type = "0", nvariates = 4),
-                          max = mcdata(c(0.2, 0.3, 0.4, 0.6), type = "0", nvariates = 4),
-                          nvariates = 4
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2, 0.3, 0.5), type = "0", nvariates = 4),
+            max = mcdata(c(0.2, 0.3, 0.4, 0.6), type = "0", nvariates = 4),
+            nvariates = 4
           ),
           data_name = "test_data",
           keys = c("category", "scenario_id")
         ),
         p_1_y = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.8, 0.7, 0.6, 0.5), type = "0", nvariates = 4),
-                          max = mcdata(c(0.9, 0.8, 0.7, 0.6), type = "0", nvariates = 4),
-                          nvariates = 4
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.8, 0.7, 0.6, 0.5), type = "0", nvariates = 4),
+            max = mcdata(c(0.9, 0.8, 0.7, 0.6), type = "0", nvariates = 4),
+            nvariates = 4
           ),
           data_name = "test_data",
           keys = c("category", "scenario_id")
         ),
         p_2 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.5, 0.6, 0.7, 0.8), type = "0", nvariates = 4),
-                          max = mcdata(c(0.6, 0.7, 0.8, 0.9), type = "0", nvariates = 4),
-                          nvariates = 4
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.5, 0.6, 0.7, 0.8), type = "0", nvariates = 4),
+            max = mcdata(c(0.6, 0.7, 0.8, 0.9), type = "0", nvariates = 4),
+            nvariates = 4
           ),
           data_name = "test_data",
           keys = c("category", "scenario_id")
@@ -242,7 +257,8 @@ suppressMessages({
     result <- trial_totals(
       mcmodule = test_module,
       mc_names = c("p_1_x", "p_1_y"),
-      trials_n = "times_n")
+      trials_n = "times_n"
+    )
 
     expect_true("p_1_all_set" %in% names(result$node_list))
     expect_true(is.null(result$node_list$sites_n))
@@ -297,7 +313,11 @@ suppressMessages({
       trials_n = "times_n",
       subsets_n = "sites_n",
       subsets_p = "p_2",
-      level_suffix = c(trial="singletime", subset="singlesite", set="allsites")
+      level_suffix = c(
+        trial = "singletime",
+        subset = "singlesite",
+        set = "allsites"
+      )
     )
 
     expect_true("p_1_x_singletime" %in% names(result$node_list))
@@ -309,7 +329,7 @@ suppressMessages({
       trials_n = "times_n",
       subsets_n = "sites_n",
       subsets_p = "p_2",
-      level_suffix = c(set="allsites")
+      level_suffix = c(set = "allsites")
     )
 
     expect_true("p_1_x_trial" %in% names(result$node_list))
@@ -344,8 +364,8 @@ suppressMessages({
     )
 
     expect_true("p_1_all_hag_set" %in% names(result$node_list))
-    expect_equal(dim(result$node_list$p_1_all$mcnode), c(1001,1,4))
-    expect_equal(dim(result$node_list$p_1_all_hag_set$mcnode), c(1001,1,2))
+    expect_equal(dim(result$node_list$p_1_all$mcnode), c(1001, 1, 4))
+    expect_equal(dim(result$node_list$p_1_all_hag_set$mcnode), c(1001, 1, 2))
 
     reset_mctable()
   })
@@ -362,15 +382,15 @@ suppressMessages({
       subsets_n = "sites_n",
       subsets_p = "p_2",
       agg_keys = "scenario_id",
-      name="p_total"
+      name = "p_total"
     )
 
     expect_true("p_1_x_hag_set" %in% names(result$node_list))
     expect_true("p_total_hag" %in% names(result$node_list))
     expect_true("p_total" %in% names(result$node_list))
 
-    expect_equal(dim(result$node_list$p_total$mcnode), c(1001,1,4))
-    expect_equal(dim(result$node_list$p_total_hag$mcnode), c(1001,1,2))
+    expect_equal(dim(result$node_list$p_total$mcnode), c(1001, 1, 4))
+    expect_equal(dim(result$node_list$p_total_hag$mcnode), c(1001, 1, 2))
 
     reset_mctable()
   })
@@ -390,7 +410,7 @@ suppressMessages({
       keep_variates = TRUE
     )
 
-    expect_equal(dim(result$node_list$p_1_all_hag_set$mcnode), c(1001,1,4))
+    expect_equal(dim(result$node_list$p_1_all_hag_set$mcnode), c(1001, 1, 4))
     expect_true(result$node_list$p_2_hag$keep_variates)
 
     reset_mctable()
@@ -401,20 +421,28 @@ suppressMessages({
     test_module <- setup_test_mcmodule()
 
     # Expect error: node not found in mcmodule
-    expect_error({
-      result <- trial_totals(
-        test_module,
-        mc_names = c("p_1_x", "nonexistent_node"),
-        trials_n = "times_n")
-    }, "nonexistent_node not found in test_module")
+    expect_error(
+      {
+        result <- trial_totals(
+          test_module,
+          mc_names = c("p_1_x", "nonexistent_node"),
+          trials_n = "times_n"
+        )
+      },
+      "nonexistent_node not found in test_module"
+    )
 
     # Expect error: node not found in mctable
-    expect_error({
-      result <- trial_totals(
-        test_module,
-        mc_names = c("p_1_x", "p_1_y"),
-        trials_n = "nonexistent_node")
-    }, "nonexistent_node not found in mctable")
+    expect_error(
+      {
+        result <- trial_totals(
+          test_module,
+          mc_names = c("p_1_x", "p_1_y"),
+          trials_n = "nonexistent_node"
+        )
+      },
+      "nonexistent_node not found in mctable"
+    )
 
     reset_mctable()
   })
@@ -424,19 +452,21 @@ suppressMessages({
     module <- list(
       node_list = list(
         p_1 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
-                          max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
-                          nvariates = 2
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
+            max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
+            nvariates = 2
           ),
           data_name = "data_x",
           keys = c("category")
         ),
         p_2 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.5, 0.6), type = "0", nvariates = 2),
-                          max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
-                          nvariates = 2
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.5, 0.6), type = "0", nvariates = 2),
+            max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
+            nvariates = 2
           ),
           data_name = "data_x",
           keys = c("category")
@@ -467,24 +497,31 @@ suppressMessages({
     expect_equal(mod4$node_list[["pre_p_all"]]$prefix, "pre")
 
     # Error on missing node
-    expect_error(at_least_one(module, c("p1", "missing")), "not found")
+    expect_error(at_least_one(module, c("p_1", "missing")), "not found")
   })
 
   test_that("generate_all_name works with suffix and errors", {
     expect_equal(generate_all_name(c("test_a", "test_b")), "test_all")
-    expect_equal(generate_all_name(c("foo_a", "foo_b"), all_suffix = "group"), "foo_group")
+    expect_equal(
+      generate_all_name(c("foo_a", "foo_b"), all_suffix = "group"),
+      "foo_group"
+    )
     expect_error(generate_all_name(c("foo_a", "bar_b")), "common prefix")
-    expect_error(generate_all_name(c("foo_all", "foo_b")), "contains '_all' suffix")
+    expect_error(
+      generate_all_name(c("foo_all", "foo_b")),
+      "contains '_all' suffix"
+    )
   })
 
   test_that("agg_totals naming options work", {
     module <- list(
       node_list = list(
-        p1 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
-                          max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
-                          nvariates = 2
+        p_1 = list(
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
+            max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
+            nvariates = 2
           ),
           data_name = "data_x",
           keys = c("category")
@@ -499,19 +536,22 @@ suppressMessages({
     )
 
     # Default agg name
-    mod1 <- agg_totals(module, "p1")
-    expect_true("p1_agg" %in% names(mod1$node_list))
+    mod1 <- agg_totals(module, "p_1")
+    expect_true("p_1_agg" %in% names(mod1$node_list))
 
     # Custom name
-    mod2 <- agg_totals(module, "p1", name = "custom_agg")
+    mod2 <- agg_totals(module, "p_1", name = "custom_agg")
     expect_true("custom_agg" %in% names(mod2$node_list))
 
     # Custom agg_suffix
-    mod3 <- agg_totals(module, "p1", agg_suffix = "sum")
-    expect_true("p1_sum" %in% names(mod3$node_list))
+    mod3 <- agg_totals(module, "p_1", agg_suffix = "sum")
+    expect_true("p_1_sum" %in% names(mod3$node_list))
 
     # Error for invalid agg_func
-    expect_error(agg_totals(module, "p1", agg_func = "invalid"), "Aggregation function")
+    expect_error(
+      agg_totals(module, "p_1", agg_func = "invalid"),
+      "Aggregation function"
+    )
 
     # Error for missing node
     expect_error(agg_totals(module, "missing"), "not found")
@@ -521,19 +561,21 @@ suppressMessages({
     module <- list(
       node_list = list(
         p_1 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
-                          max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
-                          nvariates = 2
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
+            max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
+            nvariates = 2
           ),
           data_name = "data_x",
           keys = c("category")
         ),
         p_2 = list(
-          mcnode = mcstoc(runif,
-                          min = mcdata(c(0.4, 0.5), type = "0", nvariates = 2),
-                          max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
-                          nvariates = 2
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.4, 0.5), type = "0", nvariates = 2),
+            max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
+            nvariates = 2
           ),
           data_name = "data_x",
           keys = c("category")
@@ -558,69 +600,278 @@ suppressMessages({
     expect_true("p_1_set" %in% names(mod1$node_list))
 
     # Custom name
-    mod2 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         name = "custom_trial")
+    mod2 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      name = "custom_trial"
+    )
     expect_true("custom_trial_set" %in% names(mod2$node_list))
 
     # Custom agg_suffix - no agg_keys
-    mod3 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         agg_suffix = "aggx")
+    mod3 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      agg_suffix = "aggx"
+    )
     expect_true("p_1_set" %in% names(mod3$node_list))
 
     # Custom agg_suffix - agg_keys
-    mod4 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         agg_suffix = "aggx", agg_keys = "category")
+    mod4 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      agg_suffix = "aggx",
+      agg_keys = "category"
+    )
     expect_true("p_1_aggx_set" %in% names(mod4$node_list))
 
     # Custom all_suffix
-    mod5 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         all_suffix = "tot")
+    mod5 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      all_suffix = "tot"
+    )
     expect_true("p_1_set" %in% names(mod5$node_list))
 
-    mod5 <- trial_totals(module, mc_names = c("p_1","p_2"), trials_n = "times_n",
-                         all_suffix = "tot")
+    mod5 <- trial_totals(
+      module,
+      mc_names = c("p_1", "p_2"),
+      trials_n = "times_n",
+      all_suffix = "tot"
+    )
     expect_true("p_tot_set" %in% names(mod5$node_list))
 
     # Custom prefix
-    mod6 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         prefix = "pre")
-    expect_true("pre_p_1_set"%in%names(mod6$node_list))
+    mod6 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      prefix = "pre"
+    )
+    expect_true("pre_p_1_set" %in% names(mod6$node_list))
 
     # Custom all options
-    mod7 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n", name = "custom",
-                         agg_suffix = "aggx", all_suffix = "tot", prefix = "pre")
-    expect_true("pre_custom_set_n"%in%names(mod7$node_list))
+    mod7 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      name = "custom",
+      agg_suffix = "aggx",
+      all_suffix = "tot",
+      prefix = "pre"
+    )
+    expect_true("pre_custom_set_n" %in% names(mod7$node_list))
 
     # Custom name with agg_keys
-    mod8 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         name = "custom_trial", agg_suffix = "hag", agg_keys = "category")
+    mod8 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      name = "custom_trial",
+      agg_suffix = "hag",
+      agg_keys = "category"
+    )
 
     expect_true("custom_trial_hag_set" %in% names(mod8$node_list))
 
     # Custom name with agg_keys but no agg_suffix
-    mod9 <- trial_totals(module, mc_names = "p_1", trials_n = "times_n",
-                         name = "custom_trial", agg_suffix = "", agg_keys = "category")
+    mod9 <- trial_totals(
+      module,
+      mc_names = "p_1",
+      trials_n = "times_n",
+      name = "custom_trial",
+      agg_suffix = "",
+      agg_keys = "category"
+    )
 
     expect_true("custom_trial_set" %in% names(mod9$node_list))
 
     # Error for missing node
-    expect_error(trial_totals(module, mc_names = "missing", trials_n = "times_n"), "not found")
+    expect_error(
+      trial_totals(module, mc_names = "missing", trials_n = "times_n"),
+      "not found"
+    )
   })
 
-  test_that("at_least_one match works with agg mcmodules", {
+  test_that("at_least_one match works with agg mcnodes", {
     # Create a test module with mock data
     test_module <- setup_test_mcmodule()
-    test_module <- agg_totals(test_module, c("p_1_x"), agg_keys=c("scenario_id", "category"))
-    test_module <- agg_totals(test_module, c("p_2"), agg_keys=c("scenario_id", "category"))
+    test_module <- agg_totals(
+      test_module,
+      c("p_1_x"),
+      agg_keys = c("scenario_id", "category")
+    )
+    test_module <- agg_totals(
+      test_module,
+      c("p_2"),
+      agg_keys = c("scenario_id", "category")
+    )
 
     # At least one with agg mcmodules
-    result <- at_least_one(test_module, c("p_1_x_agg", "p_2_agg"), name = "p_combined")
+    result <- at_least_one(
+      test_module,
+      c("p_1_x_agg", "p_2_agg"),
+      name = "p_combined"
+    )
 
     # Check aggregated keys
-    expect_equal(result$node_list$p_combined$agg_keys, c("scenario_id", "category"))
-
+    expect_equal(
+      result$node_list$p_combined$agg_keys,
+      c("scenario_id", "category")
+    )
   })
 
+  test_that("totals work with mcnodes with multiple data_name", {
+    # Create a test modules with mock data
+    test_module_1 <- list(
+      node_list = list(
+        p_a = list(
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
+            max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
+            nvariates = 2
+          ),
+          data_name = "data_x",
+          keys = c("category")
+        ),
+        p_b = list(
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.4, 0.5), type = "0", nvariates = 2),
+            max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
+            nvariates = 2
+          ),
+          data_name = "data_x",
+          keys = c("category")
+        ),
+        times_n = list(
+          mcnode = mcdata(c(3, 4), type = "0", nvariates = 2),
+          data_name = "data_x",
+          keys = c("category")
+        )
+      ),
+      data = list(
+        data_x = data.frame(
+          category = c("A", "B"),
+          scenario_id = c("0", "0"),
+          times_n = c(3, 4)
+        )
+      )
+    )
 
+    test_module_2 <- setup_test_mcmodule()
+
+    test_module <- combine_modules(test_module_1, test_module_2)
+
+    # At least one with agg mcmodules
+    test_module <- at_least_one(
+      test_module,
+      c("p_a", "p_2"),
+      name = "p_combined_1"
+    )
+
+    # Check both data_names are stored
+    expect_equal(
+      test_module$node_list$p_combined_1$data_name,
+      c("data_x", "test_data")
+    )
+    expect_equal(
+      test_module$node_list$p_combined_1$summary$category,
+      c("A", "B", "A", "B")
+    )
+
+    # Combine in at_least_one
+    test_module <- at_least_one(
+      mcmodule = test_module,
+      mc_names = c("p_combined_1", "p_b"),
+      name = "p_combined_2"
+    )
+
+    # Check both data_names are stored and dimensions
+    expect_equal(
+      test_module$node_list$p_combined_2$data_name,
+      c("data_x", "test_data")
+    )
+    expect_equal(
+      test_module$node_list$p_combined_2$summary$category,
+      c("A", "B", "A", "B")
+    )
+
+    # Combine in trial_totals
+    test_module <- trial_totals(
+      mcmodule = test_module,
+      mc_names = c("p_combined_1", "p_b"),
+      trials_n = "times_n",
+      name = "p_combined_3"
+    )
+
+    # Check data_names and dimensions
+    expect_equal(
+      test_module$node_list$p_combined_3_set$data_name,
+      c("data_x", "test_data")
+    )
+    expect_equal(
+      test_module$node_list$p_combined_3_set$summary$category,
+      c("A", "B", "A", "B")
+    )
+    expect_equal(dim(test_module$node_list$p_b$mcnode)[3], 2)
+    expect_equal(dim(test_module$node_list$p_b_set$mcnode)[3], 4)
+  })
+
+  test_that("trial_totals works with explicit data_name argument", {
+    # Reuse module from previous test
+    test_module <- list(
+      node_list = list(
+        p_1 = list(
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.1, 0.2), type = "0", nvariates = 2),
+            max = mcdata(c(0.2, 0.3), type = "0", nvariates = 2),
+            nvariates = 2
+          ),
+          data_name = "data_a",
+          keys = c("category")
+        ),
+        p_2 = list(
+          mcnode = mcstoc(
+            runif,
+            min = mcdata(c(0.4, 0.5), type = "0", nvariates = 2),
+            max = mcdata(c(0.6, 0.7), type = "0", nvariates = 2),
+            nvariates = 2
+          ),
+          data_name = "data_b",
+          keys = c("category")
+        ),
+        times_n = list(
+          mcnode = mcdata(c(3, 4), type = "0", nvariates = 2),
+          data_name = "data_a",
+          keys = c("category")
+        )
+      ),
+      data = list(
+        data_a = data.frame(
+          category = c("A", "B"),
+          scenario_id = c("0", "0"),
+          times_n = c(3, 4)
+        ),
+        data_b = data.frame(category = c("B", "A"), scenario_id = c("0", "0"))
+      )
+    )
+
+    expect_message(
+      result <- trial_totals(
+        mcmodule = test_module,
+        mc_names = c("p_1", "p_2"),
+        trials_n = "times_n",
+        data_name = "data_a"
+      ),
+      "Using data_name 'data_a' for node creation"
+    )
+
+    expect_equal(result$node_list$p_all_set$data_name, c("data_a", "data_b"))
+  })
 })
-
